@@ -39,27 +39,12 @@ THE SOFTWARE.
 #include "crypt.h"
 #include "config.h"
 
-#define PIANO_PROTOCOL_VERSION "28"
+#define PIANO_PROTOCOL_VERSION "29"
 #define PIANO_RPC_HOST "www.pandora.com"
 #define PIANO_RPC_PORT "80"
 #define PIANO_RPC_PATH "/radio/xmlrpc/v" PIANO_PROTOCOL_VERSION "?"
 #define PIANO_SEND_BUFFER_SIZE 10000
 
-/*	more "secure" free version
- *	@param free this pointer
- *	@param zero n bytes; 0 disables zeroing (for strings with unknown size,
- *			e.g.)
- */
-void PianoFree (void *ptr, size_t size) {
-	if (ptr != NULL) {
-		if (size > 0) {
-			/* avoid reuse of freed memory */
-			memset ((char *) ptr, 0, size);
-		}
-		free (ptr);
-	}
-}
-
 /*	initialize piano handle
  *	@param piano handle
  *	@return nothing
@@ -82,21 +67,21 @@ void PianoDestroySearchResult (PianoSearchResult_t *searchResult) {
 
 	curArtist = searchResult->artists;
 	while (curArtist != NULL) {
-		PianoFree (curArtist->name, 0);
-		PianoFree (curArtist->musicId, 0);
+		free (curArtist->name);
+		free (curArtist->musicId);
 		lastArtist = curArtist;
 		curArtist = curArtist->next;
-		PianoFree (lastArtist, sizeof (*lastArtist));
+		free (lastArtist);
 	}
 
 	curSong = searchResult->songs;
 	while (curSong != NULL) {
-		PianoFree (curSong->title, 0);
-		PianoFree (curSong->artist, 0);
-		PianoFree (curSong->musicId, 0);
+		free (curSong->title);
+		free (curSong->artist);
+		free (curSong->musicId);
 		lastSong = curSong;
 		curSong = curSong->next;
-		PianoFree (lastSong, sizeof (*lastSong));
+		free (lastSong);
 	}
 }
 
@@ -104,8 +89,8 @@ void PianoDestroySearchResult (PianoSearchResult_t *searchResult) {
  *	@param station
  */
 void PianoDestroyStation (PianoStation_t *station) {
-	PianoFree (station->name, 0);
-	PianoFree (station->id, 0);
+	free (station->name);
+	free (station->id);
 	memset (station, 0, sizeof (station));
 }
 
@@ -120,7 +105,7 @@ void PianoDestroyStations (PianoStation_t *stations) {
 		lastStation = curStation;
 		curStation = curStation->next;
 		PianoDestroyStation (lastStation);
-		PianoFree (lastStation, sizeof (*lastStation));
+		free (lastStation);
 	}
 }
 
@@ -134,18 +119,17 @@ void PianoDestroyPlaylist (PianoSong_t *playlist) {
 
 	curSong = playlist;
 	while (curSong != NULL) {
-		PianoFree (curSong->audioUrl, 0);
-		PianoFree (curSong->artist, 0);
-		PianoFree (curSong->musicId, 0);
-		PianoFree (curSong->title, 0);
-		PianoFree (curSong->userSeed, 0);
-		PianoFree (curSong->identity, 0);
-		PianoFree (curSong->stationId, 0);
-		PianoFree (curSong->album, 0);
-		PianoFree (curSong->artistMusicId, 0);
+		free (curSong->audioUrl);
+		free (curSong->artist);
+		free (curSong->musicId);
+		free (curSong->title);
+		free (curSong->userSeed);
+		free (curSong->stationId);
+		free (curSong->album);
+		free (curSong->artistMusicId);
 		lastSong = curSong;
 		curSong = curSong->next;
-		PianoFree (lastSong, sizeof (*lastSong));
+		free (lastSong);
 	}
 }
 
@@ -156,41 +140,47 @@ void PianoDestroyGenres (PianoGenre_t *genres) {
 
 	curGenre = genres;
 	while (curGenre != NULL) {
-		PianoFree (curGenre->name, 0);
-		PianoFree (curGenre->musicId, 0);
+		free (curGenre->name);
+		free (curGenre->musicId);
 		lastGenre = curGenre;
 		curGenre = curGenre->next;
-		PianoFree (lastGenre, sizeof (*lastGenre));
+		free (lastGenre);
 	}
 }
 
+/*	destroy user information
+ */
+void PianoDestroyUserInfo (PianoUserInfo_t *user) {
+	free (user->webAuthToken);
+	free (user->authToken);
+	free (user->listenerId);
+}
+
 /*	frees the whole piano handle structure
  *	@param piano handle
  *	@return nothing
  */
 void PianoDestroy (PianoHandle_t *ph) {
-	PianoFree (ph->user.webAuthToken, 0);
-	PianoFree (ph->user.authToken, 0);
-	PianoFree (ph->user.listenerId, 0);
-
+	PianoDestroyUserInfo (&ph->user);
 	PianoDestroyStations (ph->stations);
 	/* destroy genre stations */
 	PianoGenreCategory_t *curGenreCat = ph->genreStations, *lastGenreCat;
 	while (curGenreCat != NULL) {
 		PianoDestroyGenres (curGenreCat->genres);
-		PianoFree (curGenreCat->name, 0);
+		free (curGenreCat->name);
 		lastGenreCat = curGenreCat;
 		curGenreCat = curGenreCat->next;
-		PianoFree (lastGenreCat, sizeof (*lastGenreCat));
+		free (lastGenreCat);
 	}
 	memset (ph, 0, sizeof (*ph));
 }
 
-/*	destroy request, free post data. req->responseData is *not* freed here!
+/*	destroy request, free post data. req->responseData is *not* freed here, as
+ *	it might be allocated by something else than malloc!
  *	@param piano request
  */
 void PianoDestroyRequest (PianoRequest_t *req) {
-	PianoFree (req->postData, 0);
+	free (req->postData);
 	memset (req, 0, sizeof (*req));
 }
 
@@ -291,17 +281,24 @@ PianoReturn_t PianoRequest (PianoHandle_t *ph, PianoRequest_t *req,
 			assert (reqData->station->id != NULL);
 			assert (reqData->format != PIANO_AF_UNKNOWN);
 
-			/* FIXME: remove static, "magic" numbers */
 			snprintf (xmlSendBuf, sizeof (xmlSendBuf), "<?xml version=\"1.0\"?>"
 					"<methodCall><methodName>playlist.getFragment</methodName>"
 					"<params><param><value><int>%lu</int></value></param>"
+					/* auth token */
 					"<param><value><string>%s</string></value></param>"
+					/* station id */
 					"<param><value><string>%s</string></value></param>"
+					/* total listening time */
 					"<param><value><string>0</string></value></param>"
+					/* time since last session */
 					"<param><value><string></string></value></param>"
+					/* tracking code */
 					"<param><value><string></string></value></param>"
+					/* audio format */
 					"<param><value><string>%s</string></value></param>"
+					/* delta listening time */
 					"<param><value><string>0</string></value></param>"
+					/* listening timestamp */
 					"<param><value><string>0</string></value></param>"
 					"</params></methodCall>", (unsigned long) timestamp,
 					ph->user.authToken, reqData->station->id,
@@ -377,8 +374,11 @@ PianoReturn_t PianoRequest (PianoHandle_t *ph, PianoRequest_t *req,
 			snprintf (xmlSendBuf, sizeof (xmlSendBuf), "<?xml version=\"1.0\"?>"
 					"<methodCall><methodName>station.setStationName</methodName>"
 					"<params><param><value><int>%lu</int></value></param>"
+					/* auth token */
 					"<param><value><string>%s</string></value></param>"
+					/* station id */
 					"<param><value><string>%s</string></value></param>"
+					/* new name */
 					"<param><value><string>%s</string></value></param>"
 					"</params></methodCall>", (unsigned long) timestamp,
 					ph->user.authToken, reqData->station->id,
@@ -388,8 +388,8 @@ PianoReturn_t PianoRequest (PianoHandle_t *ph, PianoRequest_t *req,
 					ph->routeId, ph->user.listenerId, reqData->station->id,
 					urlencodedNewName);
 
-			PianoFree (urlencodedNewName, 0);
-			PianoFree (xmlencodedNewName, 0);
+			free (urlencodedNewName);
+			free (xmlencodedNewName);
 			break;
 		}
 
@@ -402,7 +402,9 @@ PianoReturn_t PianoRequest (PianoHandle_t *ph, PianoRequest_t *req,
 			snprintf (xmlSendBuf, sizeof (xmlSendBuf), "<?xml version=\"1.0\"?>"
 					"<methodCall><methodName>station.removeStation</methodName>"
 					"<params><param><value><int>%lu</int></value></param>"
+					/* auth token */
 					"<param><value><string>%s</string></value></param>"
+					/* station id */
 					"<param><value><string>%s</string></value></param>"
 					"</params></methodCall>", (unsigned long) timestamp,
 					ph->user.authToken, station->id);
@@ -428,7 +430,9 @@ PianoReturn_t PianoRequest (PianoHandle_t *ph, PianoRequest_t *req,
 			snprintf (xmlSendBuf, sizeof (xmlSendBuf), "<?xml version=\"1.0\"?>"
 					"<methodCall><methodName>music.search</methodName>"
 					"<params><param><value><int>%lu</int></value></param>"
+					/* auth token */
 					"<param><value><string>%s</string></value></param>"
+					/* search string */
 					"<param><value><string>%s</string></value></param>"
 					"</params></methodCall>", (unsigned long) timestamp,
 					ph->user.authToken, xmlencodedSearchStr);
@@ -436,8 +440,8 @@ PianoReturn_t PianoRequest (PianoHandle_t *ph, PianoRequest_t *req,
 					"rid=%s&lid=%s&method=search&arg1=%s", ph->routeId,
 					ph->user.listenerId, urlencodedSearchStr);
 
-			PianoFree (urlencodedSearchStr, 0);
-			PianoFree (xmlencodedSearchStr, 0);
+			free (urlencodedSearchStr);
+			free (xmlencodedSearchStr);
 			break;
 		}
 
@@ -496,12 +500,23 @@ PianoReturn_t PianoRequest (PianoHandle_t *ph, PianoRequest_t *req,
 					"<methodCall><methodName>listener.addTiredSong</methodName><params>"
 					"<param><value><int>%lu</int></value></param>"
 					"<param><value><string>%s</string></value></param>"
+					/* key */
+					"<param><value><string>%s</string></value></param>"
+					/* user seed */
+					"<param><value><string>%s</string></value></param>"
+					/* station id */
 					"<param><value><string>%s</string></value></param>"
 					"</params></methodCall>", (unsigned long) timestamp,
-					ph->user.authToken, song->identity);
+					ph->user.authToken,
+					(song->musicId == NULL) ? "" : song->musicId,
+					(song->userSeed == NULL) ? "" : song->userSeed,
+					song->stationId);
 			snprintf (req->urlPath, sizeof (req->urlPath), PIANO_RPC_PATH
-					"rid=%s&lid=%s&method=addTiredSong&arg1=%s", ph->routeId,
-					ph->user.listenerId, song->identity);
+					"rid=%s&lid=%s&method=addTiredSong&arg1=%s&arg2=%s&arg3=%s",
+					ph->routeId, ph->user.listenerId,
+					(song->musicId == NULL) ? "" : song->musicId,
+					(song->userSeed == NULL) ? "" : song->userSeed,
+					song->stationId);
 			break;
 		}
 
@@ -516,6 +531,7 @@ PianoReturn_t PianoRequest (PianoHandle_t *ph, PianoRequest_t *req,
 					"<methodCall><methodName>station.setQuickMix</methodName><params>"
 					"<param><value><int>%lu</int></value></param>"
 					"<param><value><string>%s</string></value></param>"
+					/* quick mix type */
 					"<param><value><string>RANDOM</string></value></param>"
 					"<param><value><array><data>", (unsigned long) timestamp,
 					ph->user.authToken);
@@ -542,7 +558,9 @@ PianoReturn_t PianoRequest (PianoHandle_t *ph, PianoRequest_t *req,
 			}
 			strncat (xmlSendBuf,
 					"</data></array></value></param>"
+					/* content type */
 					"<param><value><string>CUSTOM</string></value></param>"
+					/* genre */
 					"<param><value><string></string></value></param>"
 					"</params></methodCall>",
 					sizeof (xmlSendBuf) - strlen (xmlSendBuf) - 1);
@@ -569,7 +587,9 @@ PianoReturn_t PianoRequest (PianoHandle_t *ph, PianoRequest_t *req,
 			snprintf (xmlSendBuf, sizeof (xmlSendBuf), "<?xml version=\"1.0\"?>"
 					"<methodCall><methodName>station.transformShared</methodName>"
 					"<params><param><value><int>%lu</int></value></param>"
+					/* auth token */
 					"<param><value><string>%s</string></value></param>"
+					/* station id */
 					"<param><value><string>%s</string></value></param>"
 					"</params></methodCall>", (unsigned long) timestamp,
 					ph->user.authToken, station->id);
@@ -589,8 +609,11 @@ PianoReturn_t PianoRequest (PianoHandle_t *ph, PianoRequest_t *req,
 			snprintf (xmlSendBuf, sizeof (xmlSendBuf), "<?xml version=\"1.0\"?>"
 					"<methodCall><methodName>playlist.narrative</methodName>"
 					"<params><param><value><int>%lu</int></value></param>"
+					/* auth token */
 					"<param><value><string>%s</string></value></param>"
+					/* station id */
 					"<param><value><string>%s</string></value></param>"
+					/* music id */
 					"<param><value><string>%s</string></value></param>"
 					"</params></methodCall>", (unsigned long) timestamp,
 					ph->user.authToken, reqData->song->stationId,
@@ -613,8 +636,11 @@ PianoReturn_t PianoRequest (PianoHandle_t *ph, PianoRequest_t *req,
 			snprintf (xmlSendBuf, sizeof (xmlSendBuf), "<?xml version=\"1.0\"?>"
 					"<methodCall><methodName>music.getSeedSuggestions</methodName>"
 					"<params><param><value><int>%lu</int></value></param>"
+					/* auth token */
 					"<param><value><string>%s</string></value></param>"
+					/* seed music id */
 					"<param><value><string>%s</string></value></param>"
+					/* max */
 					"<param><value><int>%u</int></value></param>"
 					"</params></methodCall>", (unsigned long) timestamp,
 					ph->user.authToken, reqData->musicId, reqData->max);
@@ -633,8 +659,11 @@ PianoReturn_t PianoRequest (PianoHandle_t *ph, PianoRequest_t *req,
 			snprintf (xmlSendBuf, sizeof (xmlSendBuf), "<?xml version=\"1.0\"?>"
 					"<methodCall><methodName>station.createBookmark</methodName>"
 					"<params><param><value><int>%lu</int></value></param>"
+					/* auth token */
 					"<param><value><string>%s</string></value></param>"
+					/* station id */
 					"<param><value><string>%s</string></value></param>"
+					/* music id */
 					"<param><value><string>%s</string></value></param>"
 					"</params></methodCall>", (unsigned long) timestamp,
 					ph->user.authToken, song->stationId, song->musicId);
@@ -654,7 +683,9 @@ PianoReturn_t PianoRequest (PianoHandle_t *ph, PianoRequest_t *req,
 			snprintf (xmlSendBuf, sizeof (xmlSendBuf), "<?xml version=\"1.0\"?>"
 					"<methodCall><methodName>station.createArtistBookmark</methodName>"
 					"<params><param><value><int>%lu</int></value></param>"
+					/* auth token */
 					"<param><value><string>%s</string></value></param>"
+					/* music id */
 					"<param><value><string>%s</string></value></param>"
 					"</params></methodCall>", (unsigned long) timestamp,
 					ph->user.authToken, song->artistMusicId);
@@ -741,11 +772,6 @@ PianoReturn_t PianoRequest (PianoHandle_t *ph, PianoRequest_t *req,
 	return PIANO_RET_OK;
 }
 
-#define byteswap32(x) ((((x) >> 24) & 0x000000ff) | \
-		(((x) >> 8) & 0x0000ff00) | \
-		(((x) << 8) & 0x00ff0000) | \
-		(((x) << 24) & 0xff000000))
-
 /*	parse xml response and update data structures/return new data structure
  *	@param piano handle
  *	@param initialized request (expects responseData to be a NUL-terminated
@@ -786,9 +812,9 @@ PianoReturn_t PianoResponse (PianoHandle_t *ph, PianoRequest_t *req) {
 							timestamp = strtoul (decryptedPos, NULL, 0);
 							ph->timeOffset = realTimestamp - timestamp;
 
-							PianoFree (decryptedTimestamp, 0);
+							free (decryptedTimestamp);
 						}
-						PianoFree (cryptedTimestamp, 0);
+						free (cryptedTimestamp);
 					}
 					ret = PIANO_RET_CONTINUE_REQUEST;
 					++reqData->step;
@@ -796,6 +822,11 @@ PianoReturn_t PianoResponse (PianoHandle_t *ph, PianoRequest_t *req) {
 				}
 
 				case 1:
+					/* information exists when reauthenticating, destroy to
+					 * avoid memleak */
+					if (ph->user.listenerId != NULL) {
+						PianoDestroyUserInfo (&ph->user);
+					}
 					ret = PianoXmlParseUserinfo (ph, req->responseData);
 					break;
 			}
@@ -865,7 +896,7 @@ PianoReturn_t PianoResponse (PianoHandle_t *ph, PianoRequest_t *req) {
 				assert (reqData->station != NULL);
 				assert (reqData->newName != NULL);
 
-				PianoFree (reqData->station->name, 0);
+				free (reqData->station->name);
 				reqData->station->name = strdup (reqData->newName);
 			}
 			break;
@@ -890,7 +921,7 @@ PianoReturn_t PianoResponse (PianoHandle_t *ph, PianoRequest_t *req) {
 							ph->stations = curStation->next;
 						}
 						PianoDestroyStation (curStation);
-						PianoFree (curStation, sizeof (*curStation));
+						free (curStation);
 						break;
 					}
 					lastStation = curStation;
